---
layout: post
title: "v0.1.16 Dispatch Release"
year: 2018
---

# v0.1.16 Dispatch Release

[Download v0.1.16](https://github.com/vmware/dispatch/releases/tag/v0.1.16)

 
## Support for contexts when working with Dispatch CLI [Breaking change]
After installing Dispatch, a config file `HOME/.dispatch/config.json` is generated to store details about the Dispatch deployment.
This file includes information like Dispatch API URL, authentication details or organization name. Until recently
this meant that if you wanted to connect to multiple Dispatch deployments, you had to maintain multiple config files and shuffle them around
every time you wanted to change the current environment.

To make using multiple Dispatch deployments a little bit easier, in this release we introduce CLI contexts.
CLI context is nothing more than a group of configuration options related to a particular Dispatch deployment.
The `HOME/.dispatch/config.json` file can now store multiple contexts:
 
```json
{
    "current": "dev",
    "contexts": {
        "dev": {
            "host": "dev.dispatch.example.com",
            "port": 443,
            "scheme": "",
            "organization": "dispatch",
            "cookie": "",
            "namespace": "dispatch"
        },
        "prod": {
            "host": "prod.dispatch.example.com",
            "port": 443,
            "scheme": "",
            "organization": "dispatch",
            "cookie": "",
            "namespace": "dispatch"
        }
    }
}
```
 
Dispatch CLI allows you to list them:

```shell
$ dispatch manage context
  CONTEXT |                CONFIG
------------------------------------------------
  dev     | {
          |   "host": "dev.dispatch.example.com",
          |   "port": 443,
          |   "scheme": "",
          |   "organization": "dispatch",
          |   "cookie": "",
          |   "namespace": "dispatch"
          | }
  * prod  | {
          |   "host": "prod.dispatch.example.com",
          |   "port": 443,
          |   "scheme": "",
          |   "organization": "dispatch",
          |   "cookie": "",
          |   "namespace": "dispatch"
          | }
```
 
and set the active context:
```shell
$ dispatch manage context --set dev
Set context to dev

$ dispatch manage context
  CONTEXT |                CONFIG
------------------------------------------------
  * dev   | {
          |   "host": "dev.dispatch.example.com",
          |   "port": 443,
          |   "scheme": "",
          |   "organization": "dispatch",
          |   "cookie": "",
          |   "namespace": "dispatch"
          | }
  prod    | {
          |   "host": "prod.dispatch.example.com",
          |   "port": 443,
          |   "scheme": "",
          |   "organization": "dispatch",
          |   "cookie": "",
          |   "namespace": "dispatch"
          | }

```
  
**NOTE:** This change requires config file to be context-aware (old version of config won't work with new CLI, and vice-versa).


## CLI login now works with service accounts
Previously, `dispatch login` command only worked with OIDC users (e.g. when authentication has been configured using GitHub auth provider).
Now you can run `dispatch login --service-account test-svc-account --jwt-private-key <path-to-key-file>` and the auth data will be stored in the dispatch config file.
there is also a corresponding `dispatch logout` command to clear the credentials. Note that login takes your context into account and allows you to use
different users for different environments.


## Batch resource creation now supports all resource types
One of the handy features of Dispatch CLI is batch creation of resources, where user can specify multiple resources using YAML file, and then create them all
using `dispatch create -f file.yaml` command. In this release, the batch file supports all resource types (previously event drivers, driver types, event subscriptions and APIs were missing).

The following excerpt shows an example of a file that creates all currently supported resources:

```yaml
kind: BaseImage
name: nodejs-base
dockerUrl: dispatchframework/nodejs-base:0.0.7
language: nodejs
tags:
  - key: role
    value: test
---
kind: Image
name: nodejs
baseImageName: nodejs-base
tags:
  - key: role
    value: test
---
kind: Function
name: hello-js
sourcePath: 'nodejs/hello.js'
image: nodejs
schema:
  in:
    properties:
      name:
        minLength: 1
        pattern: "^[A-Za-z]+$"
        type: string
      place:
        minLength: 1
        pattern: "^[A-Za-z]+$"
        type: string
    required:
    - name
  out:
    properties:
      myField:
        type: string
    required:
    - myField
tags:
  - key: role
    value: test
---
kind: Secret
name: open-sesame
secrets:
  password: OpenSesame
tags:
  - key: role
    value: test
---
kind: API
name: post-hello
enabled: true
function: hello-py
methods:
  - POST
protocols:
  - https
uris:
  - /hello
tags:
  - key: role
    value : test
---
kind: DriverType
name: ticker
image: kars7e/timer:latest
tags:
  - key: role
    value : test
---
kind: Driver
name: ticker
type: ticker
config:
  - key: seconds
    value: 2
tags:
  - key: role
    value : test
---
kind: Subscription
event-type: ticker.tick
function: hello-py
name: ticker-sub
source-type: ticker
tags:
  - key: role
    value : test
---
```
