///////////////////////////////////////////////////////////////////////
// Copyright (c) 2017 VMware, Inc. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
///////////////////////////////////////////////////////////////////////

// This file is safe to edit. Once it exists it will not be overwritten

package restapi

import (
	"crypto/tls"
	"net/http"

	errors "github.com/go-openapi/errors"
	runtime "github.com/go-openapi/runtime"
	middleware "github.com/go-openapi/runtime/middleware"

	"github.com/vmware/dispatch/pkg/secrets/gen/restapi/operations"
	"github.com/vmware/dispatch/pkg/secrets/gen/restapi/operations/secret"
)

//go:generate swagger generate server --target ../pkg/secrets/gen --name Secrets --spec ../swagger/secrets.yaml --model-package v1 --skip-models --exclude-main

func configureFlags(api *operations.SecretsAPI) {
	// api.CommandLineOptionsGroups = []swag.CommandLineOptionsGroup{ ... }
}

func configureAPI(api *operations.SecretsAPI) http.Handler {
	// configure the api here
	api.ServeError = errors.ServeError

	// Set your custom logger if needed. Default one is log.Printf
	// Expected interface func(string, ...interface{})
	//
	// Example:
	// api.Logger = log.Printf

	api.JSONConsumer = runtime.JSONConsumer()

	api.JSONProducer = runtime.JSONProducer()

	api.SecretAddSecretHandler = secret.AddSecretHandlerFunc(func(params secret.AddSecretParams) middleware.Responder {
		return middleware.NotImplemented("operation secret.AddSecret has not yet been implemented")
	})
	api.SecretDeleteSecretHandler = secret.DeleteSecretHandlerFunc(func(params secret.DeleteSecretParams) middleware.Responder {
		return middleware.NotImplemented("operation secret.DeleteSecret has not yet been implemented")
	})
	api.SecretGetSecretHandler = secret.GetSecretHandlerFunc(func(params secret.GetSecretParams) middleware.Responder {
		return middleware.NotImplemented("operation secret.GetSecret has not yet been implemented")
	})
	api.SecretGetSecretsHandler = secret.GetSecretsHandlerFunc(func(params secret.GetSecretsParams) middleware.Responder {
		return middleware.NotImplemented("operation secret.GetSecrets has not yet been implemented")
	})
	api.SecretUpdateSecretHandler = secret.UpdateSecretHandlerFunc(func(params secret.UpdateSecretParams) middleware.Responder {
		return middleware.NotImplemented("operation secret.UpdateSecret has not yet been implemented")
	})

	api.ServerShutdown = func() {}

	return setupGlobalMiddleware(api.Serve(setupMiddlewares))
}

// The TLS configuration before HTTPS server starts.
func configureTLS(tlsConfig *tls.Config) {
	// Make all necessary changes to the TLS configuration here.
}

// As soon as server is initialized but not run yet, this function will be called.
// If you need to modify a config, store server instance to stop it individually later, this is the place.
// This function can be called multiple times, depending on the number of serving schemes.
// scheme value will be set accordingly: "http", "https" or "unix"
func configureServer(s *http.Server, scheme, addr string) {
}

// The middleware configuration is for the handler executors. These do not apply to the swagger.json document.
// The middleware executes after routing but before authentication, binding and validation
func setupMiddlewares(handler http.Handler) http.Handler {
	return handler
}

// The middleware configuration happens before anything, this middleware also applies to serving the swagger.json document.
// So this is a good place to plug in a panic handling middleware, logging and metrics
func setupGlobalMiddleware(handler http.Handler) http.Handler {
	return handler
}
